##############################################################################
#
# Copyright (c) 2004 TINY SPRL. (http://tiny.be) All Rights Reserved.
#                    Fabien Pinckaers <fp@tiny.Be>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import interface
import tools

import gtk
from gtk import glade

class _container(object):
	def __init__(self):
		self.cont = []
		self.col = []
		self.sg = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
		self.trans_box = []
	def new(self, col=4):
		table = gtk.Table(1, col)
		table.set_homogeneous(False)
		table.set_col_spacings(3)
		table.set_row_spacings(3)
		table.set_border_width(1)
		self.cont.append( (table, 0, 0) )
		self.col.append( col )

	def get(self):
		return self.cont[-1][0]
	def pop(self):
		(table, x, y) = self.cont.pop()
		self.col.pop()
		return table
	def newline(self):
		(table, x, y) = self.cont[-1]
		if x>0:
			self.cont[-1] = (table, 0, y+1)
		table.resize(y+1,self.col[-1])
	def wid_add(self, widget, l=1, name=None, expand=False, ypadding=2, rowspan=1, translate=False, fname=None):
		(table, x, y) = self.cont[-1]
		if l>self.col[-1]:
			l=self.col[-1]
		a = name and 1 or 0
		if l+x+a>self.col[-1]:
			self.newline()
			(table, x, y) = self.cont[-1]
		if expand:
			yopt = gtk.EXPAND | gtk.FILL
		else:
			yopt = False
		if name:
			label = gtk.Label(name)
			label.set_alignment(1.0, 0.5)
			if (self.col[-1]==4) and (x in (0,2)):
				label.set_size_request(120,-1)
				label.width_chars = 16
			table.attach(label, x, x+1, y, y+rowspan, yoptions=yopt, xoptions=0, ypadding=ypadding, xpadding=5)
			x=x+1
		hbox = widget
		hbox.show_all()
		if translate:
			hbox = gtk.HBox(spacing=3)
			hbox.pack_start(widget)
			img = gtk.Image()
			img.set_from_stock('terp-translate', gtk.ICON_SIZE_MENU)
			ebox = gtk.EventBox()
			ebox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
			self.trans_box.append((ebox, name, fname, widget))
			
			ebox.add(img)
			hbox.pack_start(ebox, fill=False, expand=False)
			hbox.show_all()
		table.attach(hbox, x, x+l, y, y+rowspan, yoptions=yopt, ypadding=ypadding, xpadding=5)
		self.cont[-1] = (table, x+l, y)
		wid_list = table.get_children()
		wid_list.reverse()
		table.set_focus_chain(wid_list)

class parser_form(interface.parser_interface):
	def parse(self, model, root_node, fields, notebook=None):
		self.dict_widget = {}
		attrs = tools.node_attributes(root_node)
		container = _container()
		container.new(col = int(attrs.get('col', 4)))

		for node in root_node.childNodes:
			if not node.nodeType==node.ELEMENT_NODE:
				continue
			attrs = tools.node_attributes(node)
			if node.localName=='separator':
				vbox = gtk.VBox()
				if 'string' in attrs:
					l = gtk.Label(attrs.get('string', 'No String Attr.'))
					l.set_alignment(0.0, 0.5)
					vbox.pack_start(l)
				vbox.pack_start(gtk.HSeparator())
				container.wid_add(vbox,attrs.get('colspan',1),expand=int(attrs.get('expand',0)), ypadding=10)
			elif node.localName=='label':
				l = gtk.Label(attrs.get('string', 'No String Attr.'))
				container.wid_add(l,expand=int(attrs.get('expand',0)))

			elif node.localName=='newline':
				container.newline()

			elif node.localName=='notebook':
				nb = gtk.Notebook()
				if attrs and 'tabpos' in attrs:
					pos = {'up':gtk.POS_TOP,
						'down':gtk.POS_BOTTOM,
						'left':gtk.POS_LEFT,
						'right':gtk.POS_RIGHT
					}[attrs['tabpos']]
				else:
					pos = gtk.POS_LEFT
				nb.set_tab_pos(pos)
				nb.set_border_width(3)
				container.wid_add(nb, attrs.get('colspan', 3), expand=True )
				self.parse(model, node, fields, nb)
			elif node.localName=='page':
				l = gtk.Label(attrs.get('string','No String Attr.'))
				result = self.parse(model, node, fields, notebook)
				notebook.append_page(result, l)

			elif node.localName=='field':
				name = attrs['name']
				del attrs['name']
				type = attrs.get('widget', fields[name]['type'])
				fields[name].update(attrs)
				fields[name]['model']=model
				if not type in widgets_type:
					print 'WIDGET %s not implemented !' % (type,)
					continue
				widget_act = widgets_type[ type ][0](self.window, self.parent, model, fields[name])
				label = None
				if fields[name].get('string',False) and not int(attrs.get('nolabel', 0)):
					label = fields[name]['string']+' :'
				self.dict_widget[name] = widget_act
				size = int(attrs.get('colspan', widgets_type[ type ][1]))
				expand = widgets_type[ type ][2]
				container.wid_add(widget_act.widget, size, label, expand, translate=fields[name].get('translate',False), fname=name)

			elif node.localName=='group':
				frame = gtk.Frame(attrs.get('string', None))
				frame.set_border_width(0)
				container.wid_add(frame, int(attrs.get('colspan', 1)), expand=int(attrs.get('expand',0)), rowspan=int(attrs.get('rowspan', 1)))
				container.new(int(attrs.get('col',4)))
				frame.add(container.get())
				if not attrs.get('string', None):
					frame.set_shadow_type(gtk.SHADOW_NONE)
					container.get().set_border_width(0)
		return container.pop()

import calendar
import spinbutton
import spinint
import char
import checkbox
import button
import reference
import binary
import textbox
import textbox_tag
import one2many
import many2many
import many2one
import selection
import one2one
import one2many_list
import picture


widgets_type = {
	'date': (calendar.calendar, 1, False),
	'time': (calendar.stime, 1, False),
	'datetime': (calendar.datetime, 1, False),
	'float': (spinbutton.spinbutton, 1, False),
	'integer': (spinint.spinint, 1, False),
	'selection': (selection.selection, 1, False),
	'char': (char.char, 1, False),
	'boolean': (checkbox.checkbox, 1, False),
	'button': (button.button, 1, False),
	'reference': (reference.reference, 1, False),
	'binary': (binary.wid_binary, 1, False),
	'picture': (picture.wid_picture, 1, False),
	'text': (textbox.textbox, 1, True),
	'text_tag': (textbox_tag.textbox_tag, 1, True),
#	'one2one': (one2one.one2one, 1, False),
#	'one2many': (one2many_list.one2many_list, 1, True),
#	'one2many_form': (one2many.one2many, 1, False),
#	'one2many_list': (one2many_list.one2many_list, 1, True),
#	'many2many': (many2many.many2many, 1, True),
	'many2one': (many2one.many2one, 1, False),
}

class form(interface.widget_interface):
	def __init__(self, window, parent, model, attrs, node, fields, triggers=[]):
		self.widgets = {}
		self.window = window
		parser = parser_form(window, parent, attrs, triggers)
		self.widget = parser.parse(model, node, fields)
		self.widgets = parser.dict_widget
		self.fields = fields
		self.model = model
		interface.widget_interface.__init__(self, window, parent, model, attrs, triggers)

	def translate(self, widget, event, model, name, src, widget_entry):
		self.trigger('translate_signal', [self.id, model, name,src, widget_entry])
		return True
		
	def _button_sig(self, widget, button):
		if not button['confirm'] or common.sur(button['confirm']):
			if button['type']=='workflow':
				self.trigger('workflow_execute', (button['function'],self.model,self.id))
			elif button['type']=='object':
				self.trigger('object_execute', (button['function'],self.model,self.id))
			elif button['type']=='action':
				self.trigger('action_execute', (button['function'],self.model,self.id))
			else:
				raise 'Unallowed button type'

	def state_propagate(self, lst, state='invalid'):
		for name in lst:
			if name in self.widgets:
				self.widgets[name].state_set(state)

	#
	# Afficher button si pas de state
	#
	def state_set(self, dynstate, objstate=None):
		interface.widget_interface.state_set(self, dynstate, objstate)
		for w in self.widgets.values():
			w.state_set(dynstate, objstate)
		for b in self.buttons:
#			b['widget'].show()
			if objstate:
				b['state']=objstate
				if (not b['states']) or (objstate in b['states']):
					b['widget'].show()
				else:
					b['widget'].hide()
			else:
				pass

	def validate(self):
		res= True
		for x in self.widgets:
			if not self.widgets[x].validate():
				logging.getLogger('view.form').info('Form not valid due to widget: '+str(x))
				res = False
		return res

	def clear(self):
		self.id=None
		for x in self.widgets.values():
			x.clear()
		self.state_set('normal', 'draft')

	def trigger_add(self, field, name, fnc, args=None):
		if field==[]:
			interface.widget_interface.trigger_add(self, field, name, fnc, args)
		else:
			f = field.pop(0)
			self.widgets[f].trigger_add(field, name, fnc, args)

	def _value_get(self):
		res = {}
		for x in self.widgets:
			if not self.widgets[x].readonly:
				res[x]=self.widgets[x].value
		return res

	def _arg_set(self, widget, value, arg):
		if widget[0] in self.widgets:
			self.widgets[widget[0]]._arg_set([], value, arg)

	def _value_set(self, value):
		if 'id' in value:
			self.id = value['id']
		for x in value:
			if x!='id':
				if x in self.widgets:
					self.widgets[x].value = value[x]
		self.state_set('normal', value.get('state','draft')) #TODO: pinky why ?

	value = property(_value_get, _value_set, None, 'The content of the form or excpetion if not valid')

	def _value_get2(self):
		res = {}
		for x in self.widgets:
			if not self.widgets[x].readonly:
				res[x]=self.widgets[x].value
		return res


