##############################################################################
#
# Copyright (c) 2004 TINY SPRL. (http://tiny.be) All Rights Reserved.
#                    Fabien Pinckaers <fp@tiny.Be>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import copy
import gtk
from gtk import glade


class parser_interface(object):
	def __init__(self, window=None, parent=None, attrs={}, triggers={}):
		self.window = window
		self.parent = parent
		self.attrs = attrs
		self.triggers = triggers

	def trigger(self, name, **args):
		if name in self.triggers:
			return self.triggers[name](**args)
		return None

	#
	# Return a widget_interface
	#
	def parse(self, model, node, fields):
		return None

_attrs_boolean = {
	'required': False,
	'readonly': False
}

class widget_interface(object):
	def _value_get(self):
		return self._value

	def _value_set(self, value):
		self._value = value

	def sig_activate(self, widget=None):
		if self.attrs.get('on_change', False):
			self.on_change(self.attrs['on_change'])

	def __init__(self, window, parent=None, model=None, attrs={}, triggers = {}):
		self._value = None
		self._triggers = triggers
		self.attrs = attrs
		self._parent = parent
		self._window = window
		for key,val in _attrs_boolean.items():
			self.attrs[key] = not attrs.get(key, False) in ('False', '0', False)
		self._default_attrs = copy.copy(attrs)
		self.model = model
		self._menu_entries = [
			('Set to default value', lambda x: self._menu_sig_default_get(), 1),
			('Set default', lambda x: self._menu_sig_default_set(), 1),
			(None, None, None),
			(gtk.STOCK_CLEAR, lambda x: self.clear(),1),
		]
		self.attrs['value'] = property(self._value_get, self._value_set, None,
			'The content of the widget or exception if not valid')

	def _readonly_set(self, value):
		pass

	def _required_set(self, value):
		pass

	def refresh(self):
		print self.attrs
		self._required_set(self.attrs.get('required', False))
		self._readonly_set(self.attrs.get('readonly', False))
		if not self.attrs.get('valid', True):
			self.color_set('invalid')
		elif self.attrs.get('readonly', False):
			self.color_set('readonly')
		elif self.attrs.get('required', False):
			self.color_set('required')
		else:
			self.color_set('normal')

	def reset(self):
		self.attrs = copy.copy(self._default_attrs)
		self.attrs['value'] = property(_value_get, _value_set, None,
			'The content of the widget or exception if not valid')
		self.refresh()

	def _color_widget(self):
		return self.widget

	def color_set(self, name):
		print 'COLOR', name
		colors = {'invalid':'#ffdddd', 'readonly':'grey', 'required':'#ddddff', 'normal':'white'}
		widget = self._color_widget()
		map = widget.get_colormap()
		colour = map.alloc_color(colors.get(name,'white'))
		style = widget.get_style().copy()
		style.base[gtk.STATE_NORMAL] = colour
		style.bg[gtk.STATE_ACTIVE] = colour
		style.text[gtk.STATE_NORMAL] = gtk.gdk.color_parse("black")
		style.fg[gtk.STATE_NORMAL] = gtk.gdk.color_parse("black")
		widget.set_style(style)

	def state_set(self, state):
		for st in self.attrs.get('states',{}).get(state,[]):
			self.attrs[st[0]] = st[1]

	def validate(self):
		if self.constraint['required'] and not self.value:
			self.attrs['valid'] = False
		self.attrs['valid'] = True
		return self.attrs['valid']

	def trigger_add(self, field, name, fnc, args=None):
		if field==[]:
			if name in self._triggers:
				self._triggers[name].append( (fnc, args) )
			else:
				self._triggers[name] = [ (fnc, args) ]

	def trigger(self, name, args):
		try:
			if name in self._triggers:
				res = None
				for x in self._triggers[name]:
					res = x[0](x[1], args)
				return res
			else:
				if self.parent:
					return self.parent.trigger(name, args)
				else:
					return None
		except rpc.rpc_exception, e:
			common.error(_('Error: ')+str(e.type), e.message, e.data)

	def _menu_sig_default_set(self):
		deps = []
		wid = self.parent.widgets
		for w in wid:
			if wid[w].change_default:
				deps.append((wid[w].attrs['string'], wid[w].attrs['name'], wid[w].value2, wid[w].value2))
		wid_common.field_pref_set(self.attrs['name'], self.attrs['string'], self.attrs['model'], self.value2, deps)

	def _menu_sig_default_get(self):
		res = rpc.session.rpc_exec_auth('/object', 'execute', self.attrs['model'], 'default_get', [self.attrs['name']])
		self.value = res.get(self.attrs['name'], False)

	def _menu_open(self, obj, event):
		obj = service.LocalService('gui.window')
		if event.button == 3:
			menu = gtk.Menu()
			for stock_id,callback,sensitivity in self._menu_entries:
				if stock_id:
					item = gtk.ImageMenuItem(stock_id)
					if callback:
						item.connect("activate",callback)
					item.set_sensitive(sensitivity)
				else:
					item=gtk.SeparatorMenuItem()
				item.show()
				menu.append(item)
			menu.popup(None,None,None,event.button,event.time)
			return True

	def _context_get(self, parent):
		arg = {}
		if self.context:
			for argument in self.context.split(','):
				(key,value) = argument.split('=')
				arg[str(key)]=self._expr_get(parent,value)
		return arg

	# this returns the domain for the field (widget)
	def _domain_get(self, parent):
		if not self.domain:
			return []
		d = parent.value2
		if parent.parent:
			d['parent'] = parent.parent.parent.value2
		return tools.expr_eval(self.domain, d)
		#FIXME: the preceding 'return' statement prevents the 5 last lines of this method to be executed
		if self.domain:
			for argument in self.domain.split(','):
				(key,value) = argument.split('=')
				arg.append((key,'=',self._expr_get(parent,value)))
		return arg

	def _expr_get(self, parent, expr):
		if expr[0]=="'":
			return expr
		return self._arg_get(parent, expr.split('.'))

	def _arg_get(self, parent, arg):
		if arg[0]=='parent':
			return self._arg_get(parent.parent.parent, arg[1:])
		elif arg[0]=='id':
			return parent.id
		elif arg[0]=='domain':
			return False
		else:
			if arg[0] in parent.value and parent.fields[arg[0]]['type'] == 'many2one':
				val = parent.value[arg[0]]
				if val:
					return val[0]
				return False
			else:
				return parent.value.get(arg[0], False)

	def _arg_set(self, widget, value, arg='value'):
		self.attrs[arg] = value
		self.refresh()

	def _focus_in(self):
		self.focus = self.value

	def _focus_out(self):
		if self.value != self.focus:
			self.sig_changed()

	def sig_changed(self):
		if self.change_default:
			res = rpc.session.rpc_exec_auth('/object', 'execute', 'ir.values', 'get', 'default', self.name+'='+str(self.value2), [(self.attrs['model'],False)], False, rpc.session.context)
			processed = {}
			for (id,name,value) in res:
				if not name in processed:
					self._arg_set(name.split('.'), value)
					processed[name] = 1
		if self.attrs.get('on_change',False):
			self.on_change(self.attrs['on_change'])

	def on_change(self, query):
		try:
			# split the query in two parts: the function name and the arguments
			res = re.compile('(.*)\((.*)\)').match(query)
			if not res:
				raise 'ERROR: Wrong on_change trigger: fnc(*args)'
			func_name = res.group(1)
			arg_names = [n.strip() for n in res.group(2).split(',')]

			# for each argument, get its value
			args = map(lambda x: self._arg_get(self.parent, x.split('.')), arg_names)
			# execute the onchange method on the server
			res2 = rpc.session.rpc_exec_auth('/object', 'execute', self.attrs['model'], func_name, False, *args)
			for key in res2.keys():
				for wid in res2[key].keys():
					self._arg_set(wid.split('.'), res2[key][wid], key)
		except rpc.rpc_exception, e:
			pass

