
from osv import fields, osv, orm

import math
from _common import rounding

class price_type(osv.osv):
	def _price_field_get(self, cr, uid, context={}):
		cr.execute('select name,field_description from ir_model_fields where model in (%s,%s) and ttype=%s', ('product.product','product.template', 'float'))
		return cr.fetchall()
	_name = "product.price.type"
	_description = "Price type"
	_columns = {
		"name" : fields.char("Price Name", size=32, required=True, translate=True) ,
		"active" : fields.boolean("Active"),
		"field" : fields.selection(_price_field_get, "Product Field", required=True),
	}
	_defaults = {
		"active": lambda *args: True ,
	}
price_type()

#----------------------------------------------------------
# Price lists
#----------------------------------------------------------

class product_pricelist_type(osv.osv):
	_name = "product.pricelist.type"
	_description = "Pricelist Type"
	_columns = {
		'name': fields.char('Name',size=64, required=True),
		'key': fields.char('Key', size=64, required=True),
	}
product_pricelist_type()


class product_pricelist(osv.osv):
	def _pricelist_type_get(self, cr, uid, context={}):
		cr.execute('select key,name from product_pricelist_type order by name')
		return cr.fetchall()
	_name = "product.pricelist"
	_description = "Pricelist"
	_columns = {
		'name': fields.char('Name',size=64, required=True),
		'active': fields.boolean('Active'),
		'type': fields.selection(_pricelist_type_get, 'Pricelist Type', required=True),
		'version_id': fields.one2many('product.pricelist.version', 'pricelist_id', 'Pricelist Versions')
	}
	_defaults = {
		'active': lambda *a: 1,
	}

	def price_get(self, cr, uid, ids, prod_id, qty, context={}):
		result = {}
		for id in ids:
			cr.execute('select * from product_pricelist_version where pricelist_id=%d and active=True order by id limit 1', (id,))
			plversion = cr.dictfetchone()
			if not plversion:
				raise 'pricelist', 'No active version for this pricelist !\nPlease create or active one.'

			cr.execute('select id,categ_id from product_template where id=(select product_tmpl_id from product_product where id=%d)', (prod_id,))
			tmpl_id,categ = cr.fetchone()
			categ_ids = []
			while categ:
				categ_ids.append(str(categ))
				cr.execute('select parent_id from product_category where id=%d', (categ,))
				categ = cr.fetchone()[0]
			if categ_ids:
				categ_where = '(categ_id in ('+','.join(categ_ids)+'))'
			else:
				categ_where = '(categ_id is null)'

			cr.execute('select * from product_pricelist_item where (product_tmpl_id is null or product_tmpl_id=%d) and ('+categ_where+' or (categ_id is null)) and price_version_id=%d and (min_quantity is null or min_quantity<=%d) order by sequence limit 1', (tmpl_id, plversion['id'], qty))
			res = cr.dictfetchone()
			if res:
				if res['base']=='other pricelist':
					if not res['base_pricelist_id']:
						price = 0
					else:
						price = self.price_get(cr, uid, [res['base_pricelist_id']], prod_id, qty)[res['base_pricelist_id']]
				else:
					price = self.pool.get('product.product').price_get(cr, uid, [prod_id], res['base'])[prod_id]
				price_limit = price

				price = price * (1.0-(res['price_discount'] or 0.0))
				price=rounding(price, res['price_round'])
				price += (res['price_surcharge'] or 0.0)
				if res['price_min_margin']:
					price = max(price, price_limit+res['price_min_margin'])
				if res['price_max_margin']:
					price = min(price, price_limit+res['price_max_margin'])
			else:
				price=False
			result[id] = price
			if 'uom' in context:
				result[id] = self.pool.get('product.uom')._compute_price(cr, uid, context['uom'], result[id])
		return result
product_pricelist()

class product_pricelist_version(osv.osv):
	_name = "product.pricelist.version"
	_description = "Pricelist Version"
	_columns = {
		'pricelist_id': fields.many2one('product.pricelist', 'Price List', required=True),
		'name': fields.char('Name', size=64, required=True),
		'active': fields.boolean('Active'),
		'items_id': fields.one2many('product.pricelist.item', 'price_version_id', 'Price List Items', required=True),
		'date_start': fields.date('Start Date'),
		'date_end': fields.date('End Date')
	}
	_defaults = {
		'active': lambda *a: 1,
	}
product_pricelist_version()

class product_pricelist_item(osv.osv):
	def _price_field_get(self,cr, uid, context={}):
		cr.execute('select field,name from product_price_type where active')
		result = cr.fetchall()
		result.append(('other pricelist','Other Pricelist'))
		return result

	_name = "product.pricelist.item"
	_description = "Pricelist item"
	_order = "sequence"
	_defaults = {
		'base': lambda *a: 'list_price',
		'min_quantity': lambda *a: 1,
		'sequence': lambda *a: 5,
	}
	_columns = {
		'name': fields.char('Name', size=64, required=True),
		'price_version_id': fields.many2one('product.pricelist.version', 'Price List Version', required=True),
		'product_tmpl_id': fields.many2one('product.template', 'Product Template'),
		'categ_id': fields.many2one('product.category', 'Product Category'),

		'min_quantity': fields.integer('Min. Quantity', required=True),
		'sequence': fields.integer('Priority', required=True),
		'base': fields.selection(_price_field_get, 'Based on', required=True),
		'base_pricelist_id': fields.many2one('product.pricelist', 'Base Price List'),

		'price_surcharge': fields.float('Price Surcharge'),
		'price_discount': fields.float('Price Discount'),
		'price_round': fields.float('Price Rounding'),
		'price_min_margin': fields.float('Price Min. Margin'),
		'price_max_margin': fields.float('Price Max. Margin'),
	}
	def product_id_change(self, cr, uid, ids, product_id, context={}):
		if not product_id:
			return {}
		prod = self.pool.get('product.product').read(cr, uid, [product_id], ['code','name'])
		if prod[0]['code']:
			return {'value': {'name': prod[0]['code']}}
		return {}
product_pricelist_item()


